@@ -9,16 +9,13 @@ pub enum MeasurementSetWriteError {
         argument: String,
         function: String,
         expected: String,
-        received: String
+        received: String,
     },
-
     // TODO: https://github.com/pkgw/rubbl/pull/148
     // #[error("{0}")]
     // RubblError(#[from] CasacoreError)
-        
 }
 
-
 #[derive(Error, Debug)]
 #[allow(clippy::upper_case_acronyms)]
 /// All the errors that can occur in file io operations
